from __future__ import absolute_import as _absolute_import

from pkg_resources import get_distribution as _get_distribution
from pkg_resources import DistributionNotFound as _DistributionNotFound

from .ext import CApiLib
from .build import build_capi

try:
    __version__ = _get_distribution('build_capi').version
except _DistributionNotFound:
    __version__ = 'unknown'



from __future__ import unicode_literals

import os
import sys
PY3 = sys.version_info > (3,)
from setuptools import Command

from ._unicode import unicode_airlock
from ._unicode import ascii_airlock

def _show_compilers():
    from distutils.ccompiler import show_compilers as sc
    sc()

class build_capi(Command, object):

    description = "build C/C++ libraries"

    user_options = [
        ('build-clib', 'b',
         "directory to build C/C++ libraries to"),
        ('build-temp', 't',
         "directory to put temporary build by-products"),
        ('debug', 'g',
         "compile with debugging information"),
        ('force', 'f',
         "forcibly build everything (ignore file timestamps)"),
        ('compiler=', 'c',
         "specify the compiler type"),
        ('inplace', 'i',
         "ignore build-lib and put compiled extensions into the source " +
         "directory alongside your pure Python modules"),
    ]

    boolean_options = ['inplace', 'debug', 'force']

    help_options = [
        ('help-compiler', None,
         "list available compilers", _show_compilers),
    ]

    def __init__(self, *args, **kwargs):
        self.build_clib = None
        self.build_temp = None

        # List of libraries to build
        self.capi_libs = None

        # Compilation options for all libraries
        self.include_dirs = None
        self.define = []
        self.undef = []
        self.debug = None
        self.force = 0
        self.inplace = 0
        self.compiler = None
        super(build_capi, self).__init__(*args, **kwargs)

    def initialize_options(self):

        self.build_clib = None
        self.build_temp = None
        self.capi_libs = []

        # Compilation options for all libraries
        self.include_dirs = None
        self.define = []
        self.undef = []
        self.debug = None
        self.force = 0
        self.inplace = 0
        self.compiler = None

    def finalize_options(self):
        self.set_undefined_options('build',
                                   ('build_lib', 'build_clib'),
                                   ('build_temp', 'build_temp'),
                                   ('compiler', 'compiler'),
                                   ('debug', 'debug'),
                                   ('force', 'force'))
        self.set_undefined_options('build_ext',
                                   ('inplace', 'inplace'))

        self.capi_libs = self.distribution.capi_libs

        if self.include_dirs is None:
            self.include_dirs = self.distribution.include_dirs or []

        if isinstance(self.include_dirs, str):
            self.include_dirs = self.include_dirs.split(os.pathsep)

    def run(self):
        if not self.capi_libs:
            return

        from distutils.sysconfig import customize_compiler
        from distutils.ccompiler import new_compiler
        self.compiler = new_compiler(compiler=self.compiler,
                                     dry_run=self.dry_run,
                                     force=self.force)
        customize_compiler(self.compiler)

        if self.include_dirs is not None:
            self.compiler.set_include_dirs(self.include_dirs)
        if self.define is not None:
            # 'define' option is a list of (name,value) tuples
            for (name, value) in self.define:
                self.compiler.define_macro(name, value)
        if self.undef is not None:
            for macro in self.undef:
                self.compiler.undefine_macro(macro)

        self.build_libraries(self.capi_libs)

    def get_library_names(self):
        # Assume the library list is valid -- 'check_library_list()' is
        # called from 'finalize_options()', so it should be!
        if not self.capi_libs:
            return None

        lib_names = []
        for lib in self.capi_libs:
            if callable(lib):
                lib = lib()
            lib_names.append(lib.name)
        return lib_names

    def build_libraries(self, capi_libs):
        from distutils.errors import DistutilsSetupError
        for lib in capi_libs:
            if callable(lib):
                lib = lib()
            sources = lib.sources
            if sources is None or not isinstance(sources, (list, tuple)):
                raise DistutilsSetupError(
                    ("in 'libraries' option (library '%s'), " +
                     "'sources' must be present and must be " +
                     "a list of source filenames") % lib.name)
            sources = [unicode_airlock(s) for s in sources]

            from distutils import log
            log.info("building '%s' library", lib.name)

            macros = lib.define_macros
            include_dirs = lib.include_dirs
            eca = lib.extra_compile_args
            if PY3:
                sources = [unicode_airlock(s) for s in sources]
                output_dir = unicode_airlock(self.build_temp)
                include_dirs = [unicode_airlock(incd) for incd in include_dirs]
                macros = [unicode_airlock(m) for m in macros]
            else:
                sources = [ascii_airlock(s) for s in sources]
                output_dir = ascii_airlock(self.build_temp)
                include_dirs = [ascii_airlock(incd) for incd in include_dirs]
                macros = [ascii_airlock(m) for m in macros]
            objects = self.compiler.compile(sources,
                                            output_dir=output_dir,
                                            macros=macros,
                                            include_dirs=include_dirs,
                                            debug=self.debug,
                                            extra_preargs=eca)

            if PY3:
                lib_name = unicode_airlock(self.get_ext_fullpath(lib.name))
                objects = [unicode_airlock(o) for o in objects]
                output_dir = u'./'
            else:
                lib_name = ascii_airlock(self.get_ext_fullpath(lib.name))
                objects = [ascii_airlock(o) for o in objects]
                output_dir = b'./'
            self.compiler.create_static_lib(objects, lib_name,
                                            output_dir=output_dir,
                                            debug=self.debug)

    def get_ext_fullpath(self, ext_name):
        """Returns the path of the filename for a given extension.

        The file is located in `build_lib` or directly in the package
        (inplace option).
        """
        ext_name = unicode_airlock(ext_name)
        fullname = self.get_ext_fullname(ext_name)
        modpath = fullname.split('.')
        filename = self.get_ext_filename(ext_name)
        filename = os.path.split(filename)[-1]

        if not self.inplace:
            # no further work needed
            # returning :
            #   build_dir/package/path/filename
            filename = os.path.join(*modpath[:-1] + [filename])
            return os.path.join(self.build_clib, filename)

        # the inplace option requires to find the package directory
        # using the build_py command for that
        package = '.'.join(modpath[0:-1])
        build_py = self.get_finalized_command('build_py')
        package_dir = os.path.abspath(build_py.get_package_dir(package))

        # returning
        #   package_dir/filename
        return os.path.join(package_dir, filename)

    def get_ext_fullname(self, ext_name):
        """Returns the fullname of a given extension name.

        Adds the `package.` prefix"""
        if not hasattr(self, 'package') or self.package is None:
            return ext_name
        else:
            return self.package + '.' + ext_name

    def get_ext_filename(self, ext_name):
        r"""Convert the name of an extension (eg. "foo.bar") into the name
        of the file from which it will be loaded (eg. "foo/bar.so", or
        "foo\bar.pyd").
        """
        # from distutils.sysconfig import get_config_var
        ext_path = ext_name.split('.')
        # OS/2 has an 8 character module (extension) limit :-(
        if os.name == "os2":
            ext_path[len(ext_path) - 1] = ext_path[len(ext_path) - 1][:8]
        # extensions in debug_mode are named 'module_d.pyd' under windows
        # so_ext = get_config_var('SO')
        so_ext = ''
        if os.name == 'nt' and self.debug:
            return os.path.join(*ext_path) + '_d' + so_ext
        return os.path.join(*ext_path) + so_ext



from __future__ import unicode_literals

from setuptools.command.build_ext import build_ext
import os

def error(msg):
    from distutils.errors import DistutilsSetupError
    raise DistutilsSetupError(msg)

class CApiLib(object):

    def __init__(self, name, sources,
                 include_dirs=None,
                 define_macros=None,
                 undef_macros=None,
                 library_dirs=None,
                 libraries=None,
                 runtime_library_dirs=None,
                 extra_objects=None,
                 extra_compile_args=None,
                 extra_link_args=None,
                 export_symbols=None,
                 depends=None):

        self.name = name
        self.sources = sources
        self.include_dirs = include_dirs or []
        self.define_macros = define_macros or []
        self.undef_macros = undef_macros or []
        self.library_dirs = library_dirs or []
        self.libraries = libraries or []
        self.runtime_library_dirs = runtime_library_dirs or []
        self.extra_objects = extra_objects or []
        self.extra_compile_args = extra_compile_args or []
        self.extra_link_args = extra_link_args or []
        self.export_symbols = export_symbols or []
        self.depends = depends or []


def capi_libs(dist, attr, value):
    assert attr == 'capi_libs'
    if isinstance(value, CApiLib) or callable(value):
        value = [value]

    for capi_lib in value:
        _check_capi_lib(capi_lib)

    cmdclass = dist.cmdclass
    _build_ext = _process_build_ext(cmdclass.pop('build_ext', build_ext))
    cmdclass['build_ext'] = _build_ext
    dist.cmdclass = cmdclass


def _check_capi_lib(capi_lib):
    if not isinstance(capi_lib, CApiLib) and not callable(capi_lib):
        error("argument to 'capi_libs=...' must be either a CApiLib or a" +
              " callable object or a list those," +
              " not %r" % (type(capi_lib).__name__,))


def _process_build_ext(klass):
    def run(self):
        self.reinitialize_command('build_capi', inplace=self.inplace,
                                  build_clib=self.build_lib)
        self.run_command("build_capi")
        return klass.old_run(self)
    klass.old_run = klass.run
    klass.run = run
    return klass



import os
import sys

from setuptools import find_packages, setup

try:
    import pypandoc
    long_description = pypandoc.convert_file('README.md', 'rst')
except (OSError, IOError, ImportError):
    long_description = open('README.md').read()


def setup_package():
    src_path = os.path.dirname(os.path.abspath(sys.argv[0]))
    old_path = os.getcwd()
    os.chdir(src_path)
    sys.path.insert(0, src_path)

    needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
    pytest_runner = ['pytest-runner'] if needs_pytest else []

    setup_requires = pytest_runner
    install_requires = []
    tests_require = ['pytest']

    metadata = dict(
        name='build-capi',
        version='1.1.10',
        maintainer="Danilo Horta",
        maintainer_email="horta@ebi.ac.uk",
        description='Build and distribute C/C++ static libraries',
        long_description=long_description,
        license="MIT",
        url='http://github.com/glimix/build-capi',
        packages=find_packages(),
        zip_safe=False,
        install_requires=install_requires,
        setup_requires=setup_requires,
        tests_require=tests_require,
        classifiers=[
            "Development Status :: 5 - Production/Stable",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python :: 2.7",
            "Programming Language :: Python :: 3.5",
            "Programming Language :: Python :: 3.6",
            "Operating System :: OS Independent",
        ],
        entry_points={
            'distutils.commands': [
                'build_capi = build_capi.build:build_capi',
            ],
            'distutils.setup_keywords': [
                'capi_libs = build_capi.ext:capi_libs',
            ],
        }, )

    try:
        setup(**metadata)
    finally:
        del sys.path[0]
        os.chdir(old_path)


if __name__ == '__main__':
    setup_package()
